@@ -120,17 +120,54 @@ void uwsgi_log_verbose(const char *fmt, ...) {
 
 
 
+static void fix_logpipe_buf(int *fd) {
+	int so_bufsize;
+        socklen_t so_bufsize_len = sizeof(int);
+
+        if (getsockopt(fd[0], SOL_SOCKET, SO_RCVBUF, &so_bufsize, &so_bufsize_len)) {
+                uwsgi_error("fix_logpipe_buf()/getsockopt()");
+		return;
+        }
+
+	if ((size_t)so_bufsize < uwsgi.log_master_bufsize) {
+		so_bufsize = uwsgi.log_master_bufsize;
+		if (setsockopt(fd[0], SOL_SOCKET, SO_RCVBUF, &so_bufsize, so_bufsize_len)) {
+                        uwsgi_error("fix_logpipe_buf()/setsockopt()");
+        	}
+	}
+
+	if (getsockopt(fd[1], SOL_SOCKET, SO_SNDBUF, &so_bufsize, &so_bufsize_len)) {
+                uwsgi_error("fix_logpipe_buf()/getsockopt()");
+                return;
+        }
+
+        if ((size_t)so_bufsize < uwsgi.log_master_bufsize) {
+                so_bufsize = uwsgi.log_master_bufsize;
+                if (setsockopt(fd[1], SOL_SOCKET, SO_SNDBUF, &so_bufsize, so_bufsize_len)) {
+                        uwsgi_error("fix_logpipe_buf()/setsockopt()");
+                }
+        }
+}
 
 // create the logpipe
 void create_logpipe(void) {
 
+	if (uwsgi.log_master_stream) {
+		if (socketpair(AF_UNIX, SOCK_STREAM, 0, uwsgi.shared->worker_log_pipe)) {
+			uwsgi_error("create_logpipe()/socketpair()\n");
+			exit(1);
+		}
+	}
+	else {
 #if defined(SOCK_SEQPACKET) && defined(__linux__)
-	if (socketpair(AF_UNIX, SOCK_SEQPACKET, 0, uwsgi.shared->worker_log_pipe)) {
+		if (socketpair(AF_UNIX, SOCK_SEQPACKET, 0, uwsgi.shared->worker_log_pipe)) {
 #else
-	if (socketpair(AF_UNIX, SOCK_DGRAM, 0, uwsgi.shared->worker_log_pipe)) {
+		if (socketpair(AF_UNIX, SOCK_DGRAM, 0, uwsgi.shared->worker_log_pipe)) {
 #endif
-		uwsgi_error("socketpair()\n");
-		exit(1);
+			uwsgi_error("create_logpipe()/socketpair()\n");
+			exit(1);
+		}
+		fix_logpipe_buf(uwsgi.shared->worker_log_pipe);
 	}
 
 	uwsgi_socket_nb(uwsgi.shared->worker_log_pipe[0]);
@@ -149,13 +186,22 @@ void create_logpipe(void) {
 	}
 
 	if (uwsgi.req_log_master) {
+		if (uwsgi.log_master_req_stream) {
+			if (socketpair(AF_UNIX, SOCK_STREAM, 0, uwsgi.shared->worker_req_log_pipe)) {
+				uwsgi_error("create_logpipe()/socketpair()\n");
+				exit(1);
+			}
+		}
+		else {
 #if defined(SOCK_SEQPACKET) && defined(__linux__)
-		if (socketpair(AF_UNIX, SOCK_SEQPACKET, 0, uwsgi.shared->worker_req_log_pipe)) {
+			if (socketpair(AF_UNIX, SOCK_SEQPACKET, 0, uwsgi.shared->worker_req_log_pipe)) {
 #else
-		if (socketpair(AF_UNIX, SOCK_DGRAM, 0, uwsgi.shared->worker_req_log_pipe)) {
+			if (socketpair(AF_UNIX, SOCK_DGRAM, 0, uwsgi.shared->worker_req_log_pipe)) {
 #endif
-			uwsgi_error("socketpair()\n");
-			exit(1);
+				uwsgi_error("create_logpipe()/socketpair()\n");
+				exit(1);
+			}
+			fix_logpipe_buf(uwsgi.shared->worker_req_log_pipe);
 		}
 
 		uwsgi_socket_nb(uwsgi.shared->worker_req_log_pipe[0]);
@@ -287,6 +333,8 @@ void logto(char *logfile) {
 
 
 void uwsgi_setup_log() {
+	
+	uwsgi_setup_log_encoders();
 
 	if (uwsgi.daemonize) {
 		if (uwsgi.has_emperor) {
@@ -1346,6 +1394,34 @@ void uwsgi_add_logchunk(int variable, int pos, char *ptr, size_t len) {
 	}
 }
 
+static void uwsgi_log_func_do(struct uwsgi_string_list *encoders, struct uwsgi_logger *ul, char *msg, size_t len) {
+	struct uwsgi_string_list *usl = encoders;
+	// note: msg must not be freed !!!
+	char *new_msg = msg;
+	size_t new_msg_len = len;
+	while(usl) {
+		struct uwsgi_log_encoder *ule = (struct uwsgi_log_encoder *) usl->custom_ptr;
+		size_t rlen = 0;
+		char *buf = ule->func(ule, new_msg, new_msg_len, &rlen);
+		if (new_msg != msg) {
+                	free(new_msg);
+        	}
+		new_msg = buf;
+		new_msg_len = rlen;
+		usl = usl->next;
+	}
+	if (ul) {
+		ul->func(ul, new_msg, new_msg_len);
+	}
+	else {
+		new_msg_len = (size_t) write(uwsgi.original_log_fd, new_msg, new_msg_len);
+	}
+	if (new_msg != msg) {
+		free(new_msg);
+	}
+
+}
+
 int uwsgi_master_log(void) {
 
         ssize_t rlen = read(uwsgi.shared->worker_log_pipe[0], uwsgi.log_master_buf, uwsgi.log_master_bufsize);
@@ -1379,7 +1455,7 @@ int uwsgi_master_log(void) {
                         if (uwsgi_regexp_match(url->pattern, url->pattern_extra, uwsgi.log_master_buf, rlen) >= 0) {
                                 struct uwsgi_logger *ul_route = (struct uwsgi_logger *) url->custom_ptr;
                                 if (ul_route) {
-                                        ul_route->func(ul_route, uwsgi.log_master_buf, rlen);
+					uwsgi_log_func_do(uwsgi.requested_log_encoders, ul_route, uwsgi.log_master_buf, rlen);
                                         finish = 1;
                                 }
                         }
@@ -1397,14 +1473,14 @@ int uwsgi_master_log(void) {
                         if (ul->id) {
                                 goto next;
                         }
-                        ul->func(ul, uwsgi.log_master_buf, rlen);
+                        uwsgi_log_func_do(uwsgi.requested_log_encoders, ul, uwsgi.log_master_buf, rlen);
                         raw_log = 0;
 next:
                         ul = ul->next;
                 }
 
                 if (raw_log) {
-                        rlen = write(uwsgi.original_log_fd, uwsgi.log_master_buf, rlen);
+			uwsgi_log_func_do(uwsgi.requested_log_encoders, NULL, uwsgi.log_master_buf, rlen);
                 }
                 return 0;
         }
@@ -1423,7 +1499,7 @@ int uwsgi_master_req_log(void) {
                         if (uwsgi_regexp_match(url->pattern, url->pattern_extra, uwsgi.log_master_buf, rlen) >= 0) {
                                 struct uwsgi_logger *ul_route = (struct uwsgi_logger *) url->custom_ptr;
                                 if (ul_route) {
-                                        ul_route->func(ul_route, uwsgi.log_master_buf, rlen);
+                                        uwsgi_log_func_do(uwsgi.requested_log_req_encoders, ul_route, uwsgi.log_master_buf, rlen);
                                         finish = 1;
                                 }
                         }
@@ -1441,14 +1517,14 @@ int uwsgi_master_req_log(void) {
                         if (ul->id) {
                                 goto next;
                         }
-                        ul->func(ul, uwsgi.log_master_buf, rlen);
+                        uwsgi_log_func_do(uwsgi.requested_log_req_encoders, ul, uwsgi.log_master_buf, rlen);
                         raw_log = 0;
 next:
                         ul = ul->next;
                 }
 
                 if (raw_log) {
-                        rlen = write(uwsgi.original_log_fd, uwsgi.log_master_buf, rlen);
+			uwsgi_log_func_do(uwsgi.requested_log_req_encoders, NULL, uwsgi.log_master_buf, rlen);
                 }
                 return 0;
         }
@@ -1511,3 +1587,361 @@ void uwsgi_threaded_logger_spawn() {
 	}
 }
 
+void uwsgi_register_log_encoder(char *name, char *(*func)(struct uwsgi_log_encoder *, char *, size_t, size_t *)) {
+	struct uwsgi_log_encoder *old_ule = NULL, *ule = uwsgi.log_encoders;
+
+	while(ule) {
+		if (!strcmp(ule->name, name)) {
+			ule->func = func;
+			return;
+		}
+		old_ule = ule;
+		ule = ule->next;
+	}
+
+	ule = uwsgi_calloc(sizeof(struct uwsgi_log_encoder));
+	ule->name = name;
+	ule->func = func;
+
+	if (old_ule) {
+		old_ule->next = ule;
+	}
+	else {
+		uwsgi.log_encoders = ule;
+	}
+}
+
+struct uwsgi_log_encoder *uwsgi_log_encoder_by_name(char *name) {
+	struct uwsgi_log_encoder *ule = uwsgi.log_encoders;
+	while(ule) {
+		if (!strcmp(name, ule->name)) return ule;
+		ule = ule->next;
+	}
+	return NULL;
+}
+
+void uwsgi_setup_log_encoders() {
+	struct uwsgi_string_list *usl = NULL;
+	uwsgi_foreach(usl, uwsgi.requested_log_encoders) {
+		char *space = strchr(usl->value, ' ');
+		if (space) *space = 0;
+		struct uwsgi_log_encoder *ule = uwsgi_log_encoder_by_name(usl->value);
+		if (!ule) {
+			uwsgi_log("log encoder \"%s\" not found\n", usl->value);
+			exit(1);
+		}
+		if (space) *space = ' ';
+		struct uwsgi_log_encoder *ule2 = uwsgi_malloc(sizeof(struct uwsgi_log_encoder));
+		memcpy(ule2, ule, sizeof(struct uwsgi_log_encoder)); 
+		// we use a copy
+		if (space) {
+			ule2->args = uwsgi_str(space+1);
+		}
+		else {
+			ule2->args = uwsgi_str("");
+		}
+		usl->custom_ptr = ule2;
+		uwsgi_log("[log-encoder] registered %s\n", usl->value);
+	}
+
+	uwsgi_foreach(usl, uwsgi.requested_log_req_encoders) {
+                char *space = strchr(usl->value, ' ');
+                if (space) *space = 0;
+                struct uwsgi_log_encoder *ule = uwsgi_log_encoder_by_name(usl->value);
+                if (!ule) {
+                        uwsgi_log("log encoder \"%s\" not found\n", usl->value);
+                        exit(1);
+                }
+                if (space) *space = ' ';
+                struct uwsgi_log_encoder *ule2 = uwsgi_malloc(sizeof(struct uwsgi_log_encoder));
+                memcpy(ule2, ule, sizeof(struct uwsgi_log_encoder));
+                // we use a copy
+		if (space) {
+                	ule2->args = uwsgi_str(space+1);
+		}
+		else {
+			ule2->args = uwsgi_str("");
+		}
+                usl->custom_ptr = ule2;
+		uwsgi_log("[log-req-encoder] registered %s\n", usl->value);
+        }
+}
+
+#ifdef UWSGI_ZLIB
+static char *uwsgi_log_encoder_gzip(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+	struct uwsgi_buffer *ub = uwsgi_gzip(msg, len);
+	if (!ub) return NULL;
+	*rlen = ub->pos;
+	// avoid destruction
+	char *buf = ub->buf;
+	ub->buf = NULL;
+	uwsgi_buffer_destroy(ub);
+	return buf;
+}
+
+static char *uwsgi_log_encoder_compress(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+	size_t c_len = (size_t) compressBound(len);
+	uLongf destLen = c_len;
+	char *buf = uwsgi_malloc(c_len);
+	if (compress((Bytef *) buf, &destLen, (Bytef *)msg, (uLong) len) == Z_OK) {
+		*rlen = destLen;
+		return buf;
+	}
+	free(buf);
+	return NULL;
+}
+#endif
+
+/*
+
+really fast encoder adding only a prefix
+
+*/
+static char *uwsgi_log_encoder_prefix(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+	char *buf = NULL;
+	struct uwsgi_buffer *ub = uwsgi_buffer_new(len + strlen(ule->args));
+	if (uwsgi_buffer_append(ub, ule->args, strlen(ule->args))) goto end;
+	if (uwsgi_buffer_append(ub, msg, len)) goto end;
+	*rlen = ub->pos;
+	buf = ub->buf;
+	ub->buf = NULL;
+end:
+	uwsgi_buffer_destroy(ub);
+	return buf;
+}
+
+/*
+
+really fast encoder adding only a newline
+
+*/
+static char *uwsgi_log_encoder_nl(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+        char *buf = NULL;
+        struct uwsgi_buffer *ub = uwsgi_buffer_new(len + 1);
+        if (uwsgi_buffer_append(ub, msg, len)) goto end;
+        if (uwsgi_buffer_byte(ub, '\n')) goto end;
+        *rlen = ub->pos;
+        buf = ub->buf;
+        ub->buf = NULL;
+end:
+        uwsgi_buffer_destroy(ub);
+        return buf;
+}
+
+/*
+
+really fast encoder adding only a suffix
+
+*/
+static char *uwsgi_log_encoder_suffix(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+        char *buf = NULL;
+        struct uwsgi_buffer *ub = uwsgi_buffer_new(len + strlen(ule->args));
+        if (uwsgi_buffer_append(ub, msg, len)) goto end;
+        if (uwsgi_buffer_append(ub, ule->args, strlen(ule->args))) goto end;
+        *rlen = ub->pos;
+        buf = ub->buf;
+        ub->buf = NULL;
+end:
+        uwsgi_buffer_destroy(ub);
+        return buf;
+}
+
+
+
+void uwsgi_log_encoder_parse_vars(struct uwsgi_log_encoder *ule) {
+		char *ptr = ule->args;
+		size_t remains = strlen(ptr);
+		char *base = ptr;
+		size_t base_len = 0;
+		char *var = NULL;
+		size_t var_len = 0;
+		int status = 0; // 1 -> $ 2-> { end -> }
+		while(remains--) {
+			char b = *ptr++;
+			if (status == 1) {
+				if (b == '{') {
+					status = 2;
+					continue;
+				}
+				base_len+=2;
+				status = 0;
+				continue;
+			}
+			else if (status == 2) {
+				if (b == '}') {
+					status = 0;
+					uwsgi_string_new_list((struct uwsgi_string_list **) &ule->data, uwsgi_concat2n(base, base_len, "", 0));
+					struct uwsgi_string_list *usl = uwsgi_string_new_list((struct uwsgi_string_list **) &ule->data, uwsgi_concat2n(var, var_len, "", 0));
+					usl->custom = 1;
+					var = NULL;
+					var_len = 0;
+					base = NULL;
+					base_len = 0;
+					continue;
+				}
+				if (!var) var = (ptr-1);
+				var_len++;
+				continue;
+			}
+			// status == 0
+			if (b == '$') {
+				status = 1;
+			}
+			else {
+				if (!base) base = (ptr-1);
+				base_len++;
+			}
+		}
+
+		if (base) {
+			if (status == 1) {
+				base_len+=2;
+			}
+			else if (status == 2) {
+				base_len+=3;
+			}
+			uwsgi_string_new_list((struct uwsgi_string_list **) &ule->data, uwsgi_concat2n(base, base_len, "", 0));
+		}
+}
+
+/*
+        // format: foo ${var} bar
+        msg (the logline)
+        msgnl (the logine with newline)
+        unix (the time_t value)
+        micros (current microseconds)
+        strftime (strftime)
+*/
+static char *uwsgi_log_encoder_format(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+	
+	if (!ule->configured) {
+		uwsgi_log_encoder_parse_vars(ule);
+		ule->configured = 1;
+	}
+
+	struct uwsgi_buffer *ub = uwsgi_buffer_new(strlen(ule->args) + len);
+	struct uwsgi_string_list *usl = (struct uwsgi_string_list *) ule->data;
+	char *buf = NULL;
+	while(usl) {
+		if (usl->custom) {
+			if (!uwsgi_strncmp(usl->value, usl->len, "msg", 3)) {
+				if (msg[len-1] == '\n') {
+					if (uwsgi_buffer_append(ub, msg, len-1)) goto end;
+				}
+				else {
+					if (uwsgi_buffer_append(ub, msg, len)) goto end;
+				}
+			}
+			else if (!uwsgi_strncmp(usl->value, usl->len, "msgnl", 5)) {
+				if (uwsgi_buffer_append(ub, msg, len)) goto end;
+			}
+			else if (!uwsgi_strncmp(usl->value, usl->len, "unix", 4)) {
+				if (uwsgi_buffer_num64(ub, uwsgi_now())) goto end;
+			}
+			else if (!uwsgi_strncmp(usl->value, usl->len, "micros", 6)) {
+				if (uwsgi_buffer_num64(ub, uwsgi_micros())) goto end;
+			}
+			else if (!uwsgi_starts_with(usl->value, usl->len, "strftime:", 9)) {
+				char sftime[64];
+                                time_t now = uwsgi_now();
+				char *buf = uwsgi_concat2n(usl->value+9, usl->len-9,"", 0);
+                                int strftime_len = strftime(sftime, 64, buf, localtime(&now));
+				free(buf);
+				if (strftime_len > 0) {
+					if (uwsgi_buffer_append(ub, sftime, strftime_len)) goto end;
+				}
+			}
+		}
+		else {
+			if (uwsgi_buffer_append(ub, usl->value, usl->len)) goto end;
+		}
+		usl = usl->next;
+	}
+	buf = ub->buf;
+	*rlen = ub->pos;
+	ub->buf = NULL;
+end:
+	uwsgi_buffer_destroy(ub);
+	return buf;
+}
+
+static char *uwsgi_log_encoder_json(struct uwsgi_log_encoder *ule, char *msg, size_t len, size_t *rlen) {
+
+        if (!ule->configured) {
+                uwsgi_log_encoder_parse_vars(ule);
+                ule->configured = 1;
+        }
+
+        struct uwsgi_buffer *ub = uwsgi_buffer_new(strlen(ule->args) + len);
+        struct uwsgi_string_list *usl = (struct uwsgi_string_list *) ule->data;
+        char *buf = NULL;
+        while(usl) {
+                if (usl->custom) {
+                        if (!uwsgi_strncmp(usl->value, usl->len, "msg", 3)) {
+				size_t msg_len = len;
+                                if (msg[len-1] == '\n') msg_len--;
+				char *e_json = uwsgi_malloc(msg_len * 2);
+				escape_json(msg, msg_len, e_json);
+				if (uwsgi_buffer_append(ub, e_json, strlen(e_json))){
+                                	free(e_json);
+                                        goto end;
+                                }
+                                free(e_json);
+                        }
+                        else if (!uwsgi_strncmp(usl->value, usl->len, "msgnl", 5)) {
+				char *e_json = uwsgi_malloc(len * 2);
+                                escape_json(msg, len, e_json);
+                                if (uwsgi_buffer_append(ub, e_json, strlen(e_json))){
+                                        free(e_json);
+                                        goto end;
+                                }
+                                free(e_json);
+                        }
+                        else if (!uwsgi_strncmp(usl->value, usl->len, "unix", 4)) {
+                                if (uwsgi_buffer_num64(ub, uwsgi_now())) goto end;
+                        }
+                        else if (!uwsgi_strncmp(usl->value, usl->len, "micros", 6)) {
+                                if (uwsgi_buffer_num64(ub, uwsgi_micros())) goto end;
+                        }
+                        else if (!uwsgi_starts_with(usl->value, usl->len, "strftime:", 9)) {
+                                char sftime[64];
+                                time_t now = uwsgi_now();
+                                char *buf = uwsgi_concat2n(usl->value+9, usl->len-9, "", 0);
+                                int strftime_len = strftime(sftime, 64, buf, localtime(&now));
+                                free(buf);
+                                if (strftime_len > 0) {
+					char *e_json = uwsgi_malloc(strftime_len * 2);
+					escape_json(sftime, strftime_len, e_json);
+                                        if (uwsgi_buffer_append(ub, e_json, strlen(e_json))){
+						free(e_json);
+						goto end;
+					}
+					free(e_json);
+                                }
+                        }
+                }
+                else {
+                        if (uwsgi_buffer_append(ub, usl->value, usl->len)) goto end;
+                }
+                usl = usl->next;
+        }
+        buf = ub->buf;
+        *rlen = ub->pos;
+        ub->buf = NULL;
+end:
+        uwsgi_buffer_destroy(ub);
+        return buf;
+}
+
+
+void uwsgi_log_encoders_register_embedded() {
+	uwsgi_register_log_encoder("prefix", uwsgi_log_encoder_prefix);
+	uwsgi_register_log_encoder("suffix", uwsgi_log_encoder_suffix);
+	uwsgi_register_log_encoder("nl", uwsgi_log_encoder_nl);
+	uwsgi_register_log_encoder("format", uwsgi_log_encoder_format);
+	uwsgi_register_log_encoder("json", uwsgi_log_encoder_json);
+#ifdef UWSGI_ZLIB
+	uwsgi_register_log_encoder("gzip", uwsgi_log_encoder_gzip);
+	uwsgi_register_log_encoder("compress", uwsgi_log_encoder_compress);
+#endif
+}
